@@ -27,6 +27,8 @@ async def main(args):
     except json.decoder.JSONDecodeError:
         tokens = {}
 
+    if args.debug:
+        logging.root.setLevel(logging.DEBUG)
     email = args.email or input("Please enter Porsche Connect email: ")
     password = args.password or getpass()
     conn = Connection(email, password, tokens=tokens, country=args.country, language=args.language)
@@ -61,6 +63,10 @@ async def main(args):
                     data = await client.getSummary(vin)
                 elif args.command == "capabilities":
                     data = await client.getCapabilities(vin)
+                elif args.command == "permissions":
+                    data = await client.getPermissions(vin)
+                elif args.command == "services":
+                    data = await client.getServices(vin)
                 elif args.command == "position":
                     data = await client.getPosition(vin)
                 elif args.command == "emobility":
@@ -81,18 +87,21 @@ async def main(args):
                     data = await client.directChargeOn(vin, model=args.model, waitForConfirmation = not args.nowait)
                 elif args.command == "directcharge-off":
                     data = await client.directChargeOff(vin, model=args.model, waitForConfirmation = not args.nowait)
-                elif args.command == 'lock' or args.command == 'unlock':
+                elif args.command == "honk":
+                    data = await client.honkAndFlash(vin, waitForConfirmation = not args.nowait)
+                elif args.command == "flash":
+                    data = await client.flash(vin, waitForConfirmation = not args.nowait)
+                elif args.command == "lock":
+                    data = await client.lock(vin, waitForConfirmation = not args.nowait)
+                elif args.command == 'unlock':
                     pin = args.pin
                     if pin is None:
                         pin = getpass("PIN code: ")
-                    if(args.command == 'lock'):
-                        data = await client.lock(vin, pin, waitForConfirmation = not args.nowait)
-                    else:
-                        data = await client.unlock(vin, pin, waitForConfirmation = not args.nowait)
+                    data = await client.unlock(vin, pin, waitForConfirmation = not args.nowait)
 
                 print(json.dumps(data, indent=2))
-    except WrongCredentials:
-        sys.exit("Wrong email or password")
+    except WrongCredentials as e:
+        sys.exit(e.message)
 
     await conn.close()
     with open(args.session_file, 'w', encoding='utf-8') as json_file:
@@ -104,9 +113,10 @@ def cli():
             "timezone": "Europe/Stockholm"}
     config.read([".porscheconnect.cfg", os.path.expanduser("~/.porscheconnect.cfg")])
     parser = argparse.ArgumentParser(description='Porsche Connect CLI.')
-    parser.add_argument('command', choices=['list', 'overview', 'maintenance', 'summary', 'capabilities', 'emobility',
+    parser.add_argument('command', choices=['list', 'services', 'permissions', 'overview', 'maintenance', 'summary', 'capabilities', 'emobility',
         'position', 'triplongterm', 'tripshortterm', 'speedalerts', 'theftalerts', 'tokens', 'lock', 'unlock',
-        'climate-on', 'climate-off', 'directcharge-on', 'directcharge-off'])
+        'climate-on', 'climate-off', 'directcharge-on', 'directcharge-off', 'honk', 'flash'])
+    parser.add_argument("-d", "--debug", dest="debug", action="store_true")
     parser.add_argument('-e', '--email', dest='email', default=config.get('porsche', 'email'))
     parser.add_argument('-p', '--password', dest='password', default=config.get('porsche', 'password'))
     parser.add_argument('-s', '--sessionfile', dest='session_file', default=config.get('porsche', 'session_file'))
